using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class BaseConfiguration<TEntity> : IEntityTypeConfiguration<TEntity>
    where TEntity : EntityBase
{
    public virtual void Configure(EntityTypeBuilder<TEntity> builder)
    {
        builder
            .HasKey(b => b.Id)
            .HasName($"pk_{builder.Metadata.GetTableName() ??
                // Set primary key for inherited types using type name
                // instead of mapped table name
                        builder.Metadata.ShortName().ToLower()}");

        builder
            .Property(b => b.Id)
            .HasColumnName("id")
            .HasColumnType("bigint")
            .UseSequence(
                $"{builder.Metadata.GetTableName() ??
                // Set sequence for inherited types using type name
                // instead of mapped table name
                        builder.Metadata.ShortName().ToLower()}_" +
                $"{builder.Property(b => b.Id).Metadata.GetColumnName()}_" + 
                "sequence");


        builder
            .Property(b => b.Guid)
            .HasColumnName("uuid")
            .HasColumnType("uuid")
            .IsRequired(true);

        builder
            .HasAlternateKey(b => b.Guid)
            .HasName(
                "altk_" +
                $"{builder.Metadata.GetTableName() ??
                // Set alternate key for inherited types using type name
                // instead of mapped table name
                        builder.Metadata.ShortName().ToLower()}_" +
                $"{builder.Property(b => b.Guid).Metadata.GetColumnName()}");
    }
}
